c	Main program for calculating angular distributions and spectra
c		This is a culmination of lots of previous programs, but this
c		version was created in March, 2006 by Paul L. Houston
c
c	Revision 1.20
c
c	This work was supported by the National Science Foundation under 
c		Grant number CHE-0548867
c
c	Acknowledgment:  I am grateful to the advice and encouragement from
c		Greg Hall, Hahkjoon Kim, Kristen Dooley, and Simon North.  Simon
c		asked me to apply this to ClO at a stage where 
c		only a few cases were clear to me. Greg often provided 
c		advice that got me back on the right track.  The calculation is described
c		in a paper by these authors and me.
c
c	Updated versions of this program are available at
c		http://people.ccmr.cornell.edu/~plh2/group/betaofnu.htm
c		The site includes a dated list of corrections that 
c		have been made.
c		The author welcomes corrections and comments at plh2@cornell.edu
c		
c
c	
c
c
c
c
c		***************************************************************
c	PROGRAM INSTRUCTIONS
c
c	Consider a diatomic molecule in an initial electronic state which is
c	excited by linearly polarized light to a dissociative electronic 
c	state and subsequently dissociates to give products at an angle 
c	theta relative to the axis of linearly polarized light. 
c	The angular distribution is	described as 
c		I(theta)= (1/4pi)(1 + Beta P2(cos(theta)) ).   
c	This program calculates both 1) the variation of Beta with wavenumber
c	and 2) a spectral simulation for an individual vibrational band of a
c	selected electronic trasition. 
c	
c
c	The program works in a Hund's case (a) basis set and expands other
c	cases in terms of these.  It decides what to do based on the 
c	spectral parameters.
c
c	Nuclear spin statistics have not been included in determining the
c	initial populations.  If desired, the function Pop(J,P,F,M) can be
c	modified to include them.  An example is presented in 
c	comments in that function. 
c
c	There is substantial input from the user needed. 
c	The needed information is all obtained by reading an external file named 
c	betaofnuin.dat, whose description is given here. Output is written
c	to a file called results.dat
c
c   * First line:
c	Any notations that you would like to appear on the first line
c	of the output file (results.dat)
c
c   * Second line:
c	A second line of any notations that you would like to appear on 
c	the second line of the output file (results.dat)
c
c   *	Third line: 
c	This line contains information on the initial electronic 
c	state.  Three numbers (F12.8) separated by commas.  
c	The first is the spin S for the initial electronic state,
c	the second is Lambda for the initial state.  The third is 
c	the parity.  A number +/- 1.0 must appear for the parity,
c	but it is used only if Lambda=0 (other states have both parities). 
c	For	example, a Double Pi state would have a line reading
c		0.5, 1.0, 1.0
c	and a Singlet Sigma Minus would have a line reading
c		0.0, 0.0, -1.0
c	Note that the program is currently set up to do the following
c	initial (or final) states:
c		Singlet Sigma (plus or minus)
c		Singlet Pi, Delta, or Phi
c		Doublet Sigma, Pi, or Delta
c		Triplet Sigma (though I think Pi and Delta are similar) 
c	Other states require modification of the program 
c		
c   *	Fourth line: 
c	Provide similar information to the first line, except
c	provided for the upper, dissociative state.	Three numbers (F12.8).
c 
c   *	Fifth line:
c	Two floating point numbers (F12.8), Jmin and Jmax.  These provide 
c	the range of initial state J values over which the results will be 
c	summed.	This range should be large enough to include all rotational 
c	levels which, when connected to the upper state, have transition
c	frequencies whose broadened lines have amplitude at frequencies 
c	within the spectral range desired in the simulation.  Recall that 
c	the contributions from these lines are weighted by their Boltzmann
c	factors.  The time the program takes depends critically on Jmax and 
c	scales roughly as 2Jmax+1.  The maximum value of Jmax that the 
c	program can currently handle is 49.  Note that the values Jmin and
c	Jmax may be either integral or half integral, depending on the 
c	electronic spin. The program does normal calculations for Jmax up
c	to 25 (24.5, if spin=.5), but it makes approximations after this.  Note
c	also that the time for running the program is strongly dependent on Jmax. 
c
c   *	Sixth line:
c	Two integers (I5) giving nangdist, the number of elements in the
c	angular distribution, and nspect, the number of elements in the 
c	spectral distribution.  For normal operation, you should set nangdist=3; 
c	this provides the best accuracy at the fastest speed.  The ONLY reason
c	not to do this is if you want to output a nice angular distribution at each
c	spectral point using icntrl=2 (see line   below).
c	The value of nspect
c	depends on the spectral resolution.  It should probably 
c	be at least large enough to give several points within each 
c	line breadth. 
c
c   * Seventh line:
c	An integer (I5) and 4 real (F12.5) numbers.  
c	The integer: ifano = 1 for Fano lineshapes, and ifano=0
c	for Lorentzian ones.
c	A real number: dnu (also called gamma), the Lorentz broadening paramater
c	Two real numbers, g and q for the Fano lineshape
c	One real number for the Temperature in K
c	NB: the program expects all five numbers, even though not all are used for
c	a particular case.  Set g and q to zero for ifano=0 or dnu to zero for
c	ifano=1
c
c   * Eighth line:
c	Two floating point numbers (F12.8) Bin and Bfin, the rotational
c	constants in cm-1 for the initial and dissociative electronic 
c	states.
c
c   * Nineth line:
c	Two floating point numbers (F12.8) Din and Dfin, the centrifugal
c	distortion constants in cm-1 for the initial and dissociative
c	electronic states.  ( You may set either to 0.0 to ignore 
c	centrifugal distortion.)  Although some number is expected here,
c	centrifugal distortion is currently ignored by the program
c	for doublets and triplets.  
c
c   * Tenth line:
c	Two floating point numbers (F12.8) SOAin and SOAfin that give the 
c	spin-orbit constants in cm-1 for the initial and dissociative
c	electronic states.  (Set these to 0.0 if they are not appropriate
c	for the electronic state, e.g. for a singlet state).	
c
c   * Eleventh line:
c	One floating point number (F12.8) that gives Tev, the electronic
c	transition energy in cm-1 for the vibrational band desired.
c	You may have to look at the energy formulas in function Energy to 
c	figure out exactly what is wanted here. 
c
c   * Twelth line:
c	Two floating point numbers (F12.8) that give startnu and range, 
c	both in cm-1, which are the starting wavenumber for the simulation
c	and the range.
c
c   * Thirteenth line:
c	Two integers (I5) ifin and iffin which tell the program which F
c	levels you would like simulated.  ifin is for the initial state
c	and iffin is for the dissociative state.
c	Values are interpreted as follows:
c		1	Do only F1
c		2	Do F1 and F2
c		3	Do F1 and F2 and F3
c		4	Do only F2
c		5	Do F2 and F3
c		6	Do only F3
c	For a singlet state, choose a value of 1
c	For a doublet sigma state, choose 2	(unless you resolve lambda doublets)
c	For a doublet pi state, note that in the case (a) limit
c		the F2 state is doublet pi 1/2 with J=N+S, while the F2 state
c		is doublet pi 3/2 with J=N-S
c	For a triplet sigma state, see 
c		B. R. Lewis, S. T. Gibson, M. Emami, and J. H. Carver, 
c		J. Quant. Spectrosc. Radiat. Transf. 40, 1 (1988).
c		J. B. Tatum and J. K. T. Watson, Can. J. Phys. 49, 2693 (1971).
c
c   * Fourteenth line:
c	Two floating point numbers (F12.8) giving the spin-rotation splitting
c	constant Gamma in cm-1.  This is usually small 
c	and can often be neglected (i.e.set equal to zero). 
c	The first number is for the initial electronic
c	state and the second is for the dissociative electronic state.  
c	The number is ignored for singlet sigma states (just put in 0.)
c
c   * Fifteenth line:
c	Two floating point numbers (F12.8) giving  
c	the "little" lambda value in 
c	for initial and dissociative state in cm-1 as 
c	used in the energy equations in
c		B. R. Lewis, S. T. Gibson, M. Emami, and J. H. Carver, 
c		J. Quant. Spectrosc. Radiat. Transf. 40, 1 (1988).
c	This number is ignored for singlet and doublet states, 
c	but two numbers 
c	are still expected to be read. (so enter 0. for such states)
c	Special note: If the first of these numbers is between 1.9 and 2.05 and
c	if the initial state is triplet sigma, the program assumes that the molecule
c	is O2 and applies the correct nuclear spin statistics.
c
c   * Sixteenth line:
c	An integer, one of the following:
c	Each digit is interpreted separately to allow combinations
c	The ones digit (passed as icntrl to program):
c		0 for normal operation
c		1 for calculating the  spectrum only.  
c			Calculating the  spectrum is no longer  
c			much faster than calculating the spectrum and the angular 
c			distribution, so this option is not particularly useful any longer.  
c		2 for outputing the angular distribtion at each point in the spectrum.
c			After each spectral element, the program writes the angular 
c			angular distribution at that element to a file called angdist.dat
c			and then pauses.  The operator can execute commands (like renaming
c			angdist.dat) and then hits return to do the next spectral element.
c	The tens digit (passed as inuc to program):
c		0 for no nuclear spin statistics (gn=1 for all states)
c		1 for ortho para in H2 in electronic ground state
c		2-9 available for user modifications in function Pop to
c			allow users to put in whatever spin statistics are needed
c			See notes in function Pop for an explanation
c	The hundreds digit (passed as ihun to program)
c		do not use; reserved for future modifications
c	Summary:  valid values are currently 0,1,2,and 10,11,12;
c		user modification can make available 20,21,22,30,31,32,etc. 
c 
c
c		End of Instructions on using Betaofnu
c		********************************************************
c
c
c	Brief Description of the Program
c
c		The program calculates Equation A2 in the paper by Kim et al. This
c	Equation has two sorts of sums, those outside the absolute value 
c	squared and those inside -  I refer to these as incoherent and coherent 
c	sums, respectively.  The incoherent sums are performed in the main program,
c	while the coherent ones are performed in the function Coherent.
c	The main program begins by reading the input file: subroutine Readinfo 
c	reads in the file, checks for common mistakes, and loads up the data into
c	common storage areas.  These commons are listed in INCLUDE files, so that 
c	if they need be changed, it is easy to do so.
c  
c		Because there are at least ten nested do loops to the program, it is 
c	important to try to minimize the number of calculations that must be done
c	in the innermost loops.  For that reason, subroutine Loadlook is used to 
c	load up a lookup table for all the rotation matrix elements, the expansion
c	coefficients, and the linear polarization amplitude coefficients that will 
c	be needed for the computation.  Thus, in the innermost do loops, most 
c	of the needed functions just look up data rather than calculating it.  This
c	means that each important function has two versions, one to actually do the 
c	calculation called in Loadlook, and one dummy function that simply looks up
c	the value when needed in the innermost loops.  For example, d(J,N,M,theta) is
c	the function that calculates the rotation matrix, and XD(J,N,M,theta) is 
c	the function used in the innermost loops that simply looks up the needed
c	data.  Ax(J,P,F,Omega,is) is a function that gives the wavefunction expansion
c	coefficients for is=0 (lower state) or is=1 (dissociative state), and 
c	Xaxl(J,P,F,Omega) and Xaxu(J,P,F,Omega) are the functions that look up 
c	the data for the lower and dissociative state, respectively.  Finally, 
c	Amplin(Jin,Min,Omin,Omfin,Jfin) calculates the linear polarization transition
c	amplitude, while XAmplin(Jin,Min,Omin,Omfin,Jfin) is the function that looks
c	up the results.  Pop(Jin,Pin,Fin) gives the initial population of a Jin,Min 
c	state, whereas XPop(Jin,Pin,Fin) looks it up.
c	Loadlook takes a bit of time at the beginning of the program,
c	but the rest of the calculation goes much more quickly once it's done.
c
c		The main program then writes out the header material to the output file, 
c	Results.dat, and proceeds to start the summations, first over the number of
c	spectral elements, nspect, then over the number of angular elements 
c	given by nangdist, then over Ji,Mi,Pi,and Fi, the initial values of J,M,
c	P (parity), and F (rotational branch).  The angular distribution is 
c	determined at each spectral element, and it is then analyzed to determine
c	beta, the anisotropy parameter, as well as the probability for absorption.
c	Then subsequent spectral elements are analyzed until the spectrum is complete.
c
c		The determination of the angular distribution requires the coherent
c	summation, performed in function Coherent.  This routine sums over all the 
c	dissociative states that can be reached from the initial J,M,P,F state.
c	First the amplitude of absorption is determined, using weighting from the 
c	the complex broadening paramater, obtained in function FL.  
c	Then the angular form of the dissociative wavefunction is determined.
c	The amplitude and wavefunction are multiplied, and the result is 
c	multiplied by it's complex conjugate to get the probability for going from the
c	selected initial state to products scattered (along the axis of the diatomic 
c	molecule) at the selected angle.
c
c		Along the way, the program needs things like the wavenumber of a 
c	transition (function Wavenumber), the term energies (function Energy),
c	the initial population (function Pop), and other helping functions.  Outside
c	of keeping track of what's summed coherently, and what incoherently, and 
c	figuring out where all the do loops start and end, the most complicated 
c	part of the program is figuring out the electronic parity of the 
c	dissociative states (done in function ax).  There are several testing
c	programs that have been used to compare the results to those published
c	elsewhere.  Thus, a number of Test programs are included in the code that 
c	are not customarily called during normal operation.  
c
c
c	Beginning of the Main Program
c
c	Data and common blocks
c
	real Jin,Min,Pin,Fin,Theta,Result
	real xnu,sintens
	integer i,m,ipar,iparx
c
	include 'Lookup.f'
	include 'Spectra.f'
	include	'Broad.f'
	include 'ReadCmn.f'
	include 'Angdist.f'
c
	itest=0
c
c
c	start main program
c	
c
c	uncover things below to run tests, otherwise comment them out
c	itest=1
c	NB: The test input files in Betaofnu/debug now need to be modified
c	to make line 7 conform with the instructions above.  These tests 
c	were run before the readimput format was changed to its current form
c	call test1
c	call test2
c	call test3
c	call test4
c	call TestDjmn
c	if (itest.eq.1) stop
c
c
c	read information from external file and interpret it
	call readinfo
c
c
c
c
c	
c	Use Lookup table and because we need so many calls
c	to DJMN(theta)
c		As a practical matter, since we will need to sum over 
c		all J values for each wavelength and at each J value 
c		we need to calculate at least a few d matrix elements,
c		and since each takes a fair amount of time to calculate them,
c		it is best to first calculate and store all the ones that
c		will be needed. Here they're stored in a common block
c		Lookup.  There's a function loadlook that loads it with
c		what's needed and another real function XD that is just like 
c		D(J,M,N,theta) but looks up the elements instead of calculating 
c		them.  Note that what's stored and looked up depends 
c		on nangdist.
c	In fact, since the program is a set of do loops nested at least ten
c		levels, anything that needs to be calculated repetitively in the last 
c		loops should be calculated beforehand.
	write (6,*) '  Calculating D elements,etc., Please wait ...'
	call loadlook
	write (6,*) '  Done D elements, etc., Starting Spectrum'
c	call testload
c	stop
c
c
c
c	open file for writing results
	open (7,file='results.dat')
c	write out some stuff at the beginning of the output file
	call writehead

c	 
c
c	this is the main loop for the spectrum
	Do 101 m=1,nspect
	xnu=startnu+(float(m-1)*range/(float(nspect-1)))
c	show an odometer to indicate how far we've gotten	
	write(6,474) m,nspect
474	format('+',' ispect=',I4,'/',I4)
c
c
c	this is the loop over angular elements to get angular
c		distribution at this spectral element
	do 100 i=1,nangdist,1
c	determine angle at this index
	theta=3.14159265358979*float(i-1)/float(nangdist-1)
c
c	set sum to be used to sum contributions at this angle
	sum=0.
	if (icntrl.eq.2) open(8,file='angdist.dat')
c
c
c
c	these are the loops summing over all the possible initial 
c	states Jin,Min,Pin,Fin.  If the state is some linear combination
c	of omega levels, that will be dealt with in Coherent.
c
c	The value of Jcalc comes from readinfo, see notes there
c	Jcalc is the smaller of Jmax and either 25 or 24.5 
c	sum over Jin
	do 50 Jin=Jmin,Jcalc,1.
c
c
c	sum over Min
	do 50 Min= -Jin,Jin,1.
c
c	sum over parity states
c	find out how many and, if only one,what it is
c	Pin is returned only if iparx=1
	call igetpar(Jin,Pin,iparx)
	do 50 ipar=1,iparx 
	if ((iparx.eq.2).and.(ipar.eq.1)) Pin=1.0
	if ((iparx.eq.2).and.(ipar.eq.2)) Pin=-1.0
c
c	sum over F levels
	do 50 Fin=Fstart,Fend,1.
c
c
c
c
c	the sum over coherently excited intermediate states
c		is done in the function Coherent, which also multiplies
c		the result by it's c.c. and returns the probability 
c		for this wavenumber (xnu) of going from the initial  
c		state to products scattered at the specified theta.
	Result=XPop(Jin,Pin,Fin)*
	&    Coherent(Jin,Min,Pin,Fin,Theta,xnu)
	sum=sum+result
50	continue
c
c
	angdist(i)=sum
c	write (6,*) ' i,angdist(i)= ',i,angdist(i)
	if (icntrl.eq.2) write (8,*) theta,angdist(i)
c
c
c	this is the end of the angular distribution loop
c	go back for next angle or finish up this spectral element
100	continue
c
c
c
c
c	At this point we make the correction for line intensities and populations
c	in the event that Jmax is greater than Jcalc
c	This basically adjusts the magnitude of angdist (though not the 
c	actual angular distribution) for the lines not yet considered
	if (Jmax.gt.Jcalc) then
		call HighJ(xnu)
	endif
c
c
	if (icntrl.eq.2) close(8)
	if (icntrl.eq.2) pause
c
c
c 
c
c	at each point in the spectrum calculate beta
c		from the angular distribution and then integrate
c		the angular distribution over sin theta d theta
c		to get the intensity at that wavenumber.
	if (icntrl.ne.1) beta=fitbeta()
	if (icntrl.eq.1) beta=0.
	sintens=angint()
c
c
c	write the results to the output file
c	convert xnu to nm to have it both ways
	write(7,156) 1.0e07/xnu,beta,sintens,xnu
156	format(3x,f12.4,2x,f10.4,2x,e9.3,2x,F12.3)
c
c	return to do the next spectral element or finish
101	continue
c
c	close output
	close(7)
	stop
	end
c
c
c	  *********************************************
	subroutine loadlook	
c	this subroutine loads common Lookup with all the 
c	d matrix elements, amplitudes, and amplin elements
c	that will be needed for the calculation.
c	This way, they're only calculated once.
c	The routine is a bit wasteful of space, but has the virtue of 
c	simplicity
	real J,M,N,Jxstart,Nf,Jf,Fi,Pi
	integer i,ij,im,in,inf,ijf,ip,iff
c	
	include 'Lookup.f'
	include 'Spectra.f'
	include 'ReadCmn.f'
c
c	initialize matrix
	do 5 ij=1,30
	do 5 in=1,11
	do 5 im=1,70
	do 5 i=1,100
5	Look(ij,in,im,i)=0.
c
c
c
c	load it
	Jxstart=Jmin-1.
	If (Jxstart.lt.0) Jxstart=Jmin
	add=0.
	If (instate.eq.3) add=.5
	do 20 J=Jxstart,Jcalc+1.,1.
	ij=int(J+add+1.)
	do 20 N=-Ommaxfin,Ommaxfin,1.
	in=int(N+Ommaxfin+1.)
	do 20 M=-J,J,1.
	im=int(M+J+1.)
	do 20 i=1,nangdist,1
	theta=3.14159265358979*float(i-1)/float(nangdist-1)
	Look(ij,in,im,i)=d(J,N,M,theta)
20	continue
c
	do 30 J=Jxstart,Jmax+1.,1.
	ij=int(J+add+1.)
	do 30 M=-J,J,1.
	im=int(M+J+1.)
	do 30 N=-Ommaxin,Ommaxin,1.
	in=int(N+Ommaxin+1.)
	do 30 Nf=-Ommaxfin,Ommaxfin,1.
	inf=int(Nf+Ommaxfin+1.)
	do 30 Jf=J-1.,J+1.,1.
	if (Jf.lt.0) go to 30
	ijf=int(Jf+1.-(J-1.))
30	Amp(ij,im,in,inf,ijf)=Amplin(J,M,N,Nf,Jf)
c
	do 40 J=Jxstart,Jmax+1,1.
	ij=int(J+add+1.)
	do 40 Pi=-1.,1.,2
	ip=int(Pi+2.)
	do 40 Fi=Fstart,Fend,1.
	iff=int(Fi)
	do 40 N=-Ommaxin,Ommaxin,1.
	in=int(N+Ommaxin+1.)
40	Axl(ij,ip,iff,in)=Ax(J,Pi,Fi,N,0)
c
	do 50 J=Jxstart,Jmax+1,1.
	ij=int(J+add+1.)
	do 50 Pi=-1.,1.,2
	ip=int(Pi+2.)
	do 50 Fi=Ffstart,Ffend,1.
	iff=int(Fi)
	do 50 N=-Ommaxfin,Ommaxfin,1.
	in=int(N+Ommaxfin+1.)
50	Axu(ij,ip,iff,in)=Ax(J,Pi,Fi,N,1)
c
c
c
	do 60 J=Jxstart,Jmax,1
	ij=int(J+add+1.)
	do 60 Pi=-1.,1.,2
	ip=int(Pi+2.)
	do 60 Fi=Fstart,Fend,1.
	iff=int(Fi)
60	upop(ij,ip,iff)=Pop(J,Pi,Fi)
c
c
c
	return
	end
c
c
c	**********************************************
	real function Xpop(J,Pi,Fi)
	real J,Pi,Fi
	integer ij,ip,iff
c	
	include 'Lookup.f'
	include 'Spectra.f'
c	
c	This is a look alike for Pop(J,P,F)
c
	ij=int(J+add+1.)
	ip=int(Pi+2.)
	iff=int(Fi)
c
	XPop=upop(ij,ip,iff)
	return
	end
c
c
c
c	**********************************************
c
	real function Xamplin(J,M,N,Nf,Jf)
	real J,M,N,Nf,Jf
	integer ijf,inf,ij,im,in
c
c	
	include 'Lookup.f'
	include 'Spectra.f'
c	
c	This is a look alike function for Amplin that uses stored values
	ij=int(J+add+1.)
	im=int(M+J+1.)
	in=int(N+Ommaxin+1.)
	inf=int(Nf+Ommaxfin+1.)
	ijf=int(Jf+1.-(J-1.))
	Xamplin=Amp(ij,im,in,inf,ijf)
	return
	end
c
c
c	*********************************************
c
	real function XAxl(J,Pi,Fi,N)
	real J, Pi, Fi, N
	integer ij,ip,iff,in
c	
	include 'Lookup.f'
	include 'Spectra.f'
c	
c	
c	This is a look alike function for ax(J,Pi,Fi,N,0) that uses 
c	stored values
   	ij=int(J+add+1.)
	ip=int(Pi+2.)
	iff=int(Fi)
	in=int(N+Ommaxin+1.)
	Xaxl=Axl(ij,ip,iff,in)
	return
	end
c
c	**********************************************
c
	real function XAxu(J,Pi,Fi,N)
	real J,Pi,Fi,N
	integer ij,ip,iff,in
c	
	include 'Lookup.f'
	include 'Spectra.f'
c	
c	
c	This is a look alike function for ax(J,Pi,Fi,N,1) that uses
c	stored values
	ij=int(J+add+1.)
	ip=int(Pi+2.)
	iff=int(Fi)
	in=int(N+Ommaxfin+1.)
	Xaxu=Axu(ij,ip,iff,in)
	return
	end
c
c	 ******************************************
c	real function XD(J,N,M,theta)
	real function XD(J,N,M,theta)
c
c	This function is a look alike for d(J,N,M,theta) that
c		can be used in it's place.  It looks up the previously
c		calculated values of the d matrix elements and returns
c		the one requested
	real J,M,N,theta
	integer i,ij,im,in
c	
	include 'Lookup.f'
	include 'Spectra.f'
c	
c	write (6,*) '  Entering XD',d(J,N,M,theta)
	XD=0.
	If(J.gt.26.1) then
		write(6,*) ' stopped in function XD, Ji>26'
		stop
	endif
c	check to see that M and N are smaller than or equal to J
	If (Abs(M).gt.J) go to 200
	If (Abs(N).gt.J) go to 200

c	ij=int(2.*J)
c	im=int(2.*M)+int(2.*J)+1
c	in=int(2.*N)+int(2.*Ommaxfin)+1
	ij=int(J+add+1.)
	in=int(N+Ommaxfin+1.)
	im=int(M+J+1.)
	i=int(theta*(1./(3.14159265358979))*float(nangdist-1)+1.01)
	XD=Look(ij,in,im,i)
200	continue
	return
	end
c
c
c		***********************************************************
	subroutine igetpar(Jin,Pin,iparx)
	real Jin,Pin
	integer iparx
c
	include 'Spectra.f'
c
	iparx=1
	If (instate.ne.1) iparx=2
	If (instate.eq.1) then
		if (Pein.eq.1.0)  Pin=(-1.)**(Jin)
c		the case above is also called an "e" level
		if (Pein.eq.-1.0) Pin=(-1.)**(Jin+1.)
c		the case above is also called an "f" level
c		Zare eq. (48) p. 308
	endif
	return
	end
c		*******************************************************
	subroutine readinfo
	real Jin,Min,calcs,step,stepnm,bpar
c
	include 'ReadCmn.f'
	include 'Spectra.f'
	include 'Broad.f'
	include 'Lookup.f'
c
c
c	read information from external file and interpret it
	open (8,file='betaofnuin.dat')
	read (8,649) Title1
	read (8,649) Title2
	read (8,648) Sin,Lamin,Pein
	read (8,648) Sfin,Lamfin,Pefin
	read (8,650) Jmin,Jmax
	read (8,651) nangdist,nspect
	read (8,654) ifano,dnu,g,q,T
	read (8,650) Bin,Bfin
	read (8,650) Din,Dfin
	read (8,650) SOAin,SOAfin
	read (8,652) Te
	read (8,650) startnu,range
	read (8,651) ifin,iffin
	read (8,650) gamin,gamfin
	read (8,650) llamin,llamfin
	read (8,653) icntrl
648	format(3(F12.8))
649	format(A80)
650	format(2(F12.8))
651	format(2(I5))
652	format(F12.8)
653	format(I5)
654	format(I5,4(F12.5))
	close (8)
c	check Jmax
	if (Jmax.gt.50) then
		write (6,*) ' Jmax above 50 not implemented'
		write (6,*) ' dimensions of Look need to be increased'
		write (6,*) ' stopped in readinfo'
		stop
	endif
c	check nangdist
	if (nangdist.gt.100) then
		write (6,*) ' nangdist above 100 not implemented'
		write (6,*) ' dimensions of Look need to be increased'
		write (6,*) 
	&	' dimension of angdist in main may need to be increased'
		write (6,*) ' stopped in readinfo'
		stop
	endif
c
c	interpret to get instate and finstate
	If ((Sin.eq.0.).and.(Lamin.eq.0.)) then
c		singlet sigma
		instate=1
		Ommaxin=0.
	Elseif ((Sin.eq.0.).and.(Lamin.eq.1.)) then
c		singlet pi
		instate=2
		Ommaxin=1.
	Elseif ((Sin.eq.0.).and.(Lamin.eq.2.)) then
c		singlet delta
		instate=2
		Ommaxin=2.
	Elseif ((Sin.eq.0.).and.(Lamin.eq.3.)) then
c		singlet phi
		instate=2
		Ommaxin=3.
	Elseif (Sin.eq.0.5) then
c		doublet states, sigma,pi,and delta
		instate=3
		Ommaxin=Lamin+Sin
		if (Lamin.gt.2.) then
			write(6,*) ' Lamin too large in readinfo pgm'
			stop
		endif
	Elseif (Sin.eq.1.) then
c		triplet sigma state
		instate=4
		Ommaxin=Lamin+Sin
		if (Lamin.gt.0.) then
			write(6,*) ' Lamin too large in readinfo pgm'
			stop
		endif
	Else
		write (6,*) ' Stopped in main, bad input file'
		stop
	Endif
	If ((Sfin.eq.0.).and.(Lamfin.eq.0.)) then
c		singlet sigma
		finstate=1
		Ommaxfin=0.
	Elseif ((Sfin.eq.0.).and.(Lamfin.eq.1.)) then
c		singlet pi
		finstate=2
		Ommaxfin=1.
	Elseif ((Sfin.eq.0.).and.(Lamfin.eq.2.)) then
c		singlet delta
		finstate=2
		Ommaxfin=2.
	Elseif ((Sfin.eq.0.).and.(Lamfin.eq.3.)) then
c		singlet phi
		finstate=2
		Ommaxfin=3.
	Elseif (Sfin.eq.0.5) then
c		doublet sigma, pi, delta
		finstate=3
		Ommaxfin=Lamfin+Sfin
		if (Lamfin.gt.2.) then
			write(6,*) ' Lamfin too large in readinfo pgm'
			stop
		endif
	Elseif (Sfin.eq.1.) then
c		triplet sigma
		finstate=4
		Ommaxfin=lamfin+Sfin
		if (Lamfin.gt.0.) then
			write(6,*) ' Lamfin too large in readinfo pgm'
			stop
		endif
	Else
		write (6,*) ' Stopped in readinfo, bad input file'
		stop
	Endif
c
c	interpret ifin
	if (ifin.eq.1) then
		Fstart=1.
		Fend=1.
	elseif (ifin.eq.2) then
		Fstart=1.
		Fend=2.
	elseif (ifin.eq.3) then
		Fstart=1.
		Fend=3.
	elseif (ifin.eq.4) then
		Fstart=2.
		Fend=2.
	elseif (ifin.eq.5) then
		Fstart=2.
		Fend=3.
	elseif (ifin.eq.6) then
		Fstart=3.
		Fend=3.
	else
		write (6,*) ' Problem with ifin in input file'
		write (6,*) ' stopped in readinfo'
		stop
	endif
c	interpret iffin
c	interpret to get Ffstart,Ffend for dissociative state
	if (iffin.eq.1) then
		Ffstart=1.
		Ffend=1.
	elseif (iffin.eq.2) then
		Ffstart=1.
		Ffend=2.
	elseif (iffin.eq.3) then
		Ffstart=1.
		Ffend=3.
	elseif (iffin.eq.4) then
		Ffstart=2.
		Ffend=2.
	elseif (iffin.eq.5) then
		Ffstart=2.
		Ffend=3.
	elseif (iffin.eq.6) then
		Ffstart=3.
		Ffend=3.
	else
		write (6,*) ' Problem with iffin in input file'
		write (6,*) ' stopped in readinfo'
		stop
	endif
c
c
c	check on electronic parity
	if (instate.eq.1)then
		if (Pein.eq.1.) then
			if (finstate.eq.1) then
				if (Pefin.ne.1.) then
				write(6,*) 'bad combo of Pein&Pefin'
				write(6,*) ' stopped in subroutine readinfo'
				stop
				endif
			endif
		elseif (Pein.eq.-1.) then
			if (finstate.eq.1) then
				if (Pefin.ne.-1.) then
				write(6,*) 'bad combo of Pein&Pefin'
				write(6,*) ' stopped in subroutine readinfo'
				stop
				endif
			else
				Pefin=Pein
			endif
		endif
	else
		Pefin=Pein
	endif
c
c	check spin conservation
	If (Sin.ne.Sfin) then
		write(6,*) ' Violates spin conservation'
		write(6,*) ' stopped in subroutine readinfo'
		stop
	endif
c	special signal to do nuclear spin stuff for O2 in 3Sigma
	iO2=0
	If ( (Sin.eq.1.).and.(Lamin.eq.0.).and.(llamin.gt.1.9).and.
	&(llamin.lt.2.05) ) iO2=1
c	
c
c
c
c	icntrl: need to interpret digits of a (potentially) three digit integer
c	hundreds digit first
	ihun=0
	inuc=0
	if ((icntrl.ge.100).and.(icntrl.le.999)) then
		ihun=int(icntrl/100)
c		hundreds digit not implemented; save for future use
		icntrl=icntrl-ihun*100
	endif
c	now do tens digit
	if ((icntrl.ge.10).and.(icntrl.le.99)) then
		inuc=int(icntrl/10)
		icntrl=icntrl-inuc*10
	endif
c	now do ones digit
	if ((icntrl.ge.0).and.(icntrl.le.9)) then
		If (icntrl.eq.1) nangdist=2
		If ((icntrl.eq.0).or.(icntrl.eq.1).or.(icntrl.eq.2)) then
			continue
		else
			write (6,*) ' bad input for icntrl'
			write (6,*) ' stopped in readinfo'
			stop
		endif
	endif
c	we now have ihun (not used yet), inuc (used for nuclear spin statistics)
c	and icntrl (used to control how program functions)
c
c	now check for right values of Jmin and Jmax
	if ((instate.eq.1).or.(instate.eq.2).or.(instate.eq.4)) then
c	singlets or triplets, J values should be integral
		if (int(Jmin).ne.Jmin) then
			write (6,*) ' values for Jmin,Jmax should be integral'
			write (6,*) ' stopped in readinfo'
			stop
		elseif (int(Jmax).ne.Jmax) then
			write (6,*) ' values for Jmin,Jmax should be integral'
			write (6,*) ' stopped in readinfo'
			stop
		endif
	elseif (instate.eq.3) then
c	doublets, J values should be half integral
		if (int(Jmin).ne.int(Jmin-.5)) then
			write (6,*) ' Jmin,Jmax values should be half integral'
			write (6,*) ' stopped in readinfo'
			stop
		elseif (int(Jmax).ne.int(Jmax-.5)) then
			write (6,*) ' Jmin,Jmax vlaues should be half integral'
			write (6,*) ' stopped in readinfo'
			stop
		endif
	endif
c
c
c	Use approximation for high J
c	There is a problem for high J because d(J,N,M,theta) does
c	not work well for J above  25 (24.5).  However, by the time
c	J is this high, the angular distribution is not substantially 
c	different from that for J=25 (24.5).  The amplitude of the transition,
c	needs to be adjusted from that for J=25 (24.5); we do that below.
c	but we approximate the angular distribution as that for J=25 (24.5).
c	The first step is to find the max value to sum normally to
	Jcalc=Jmax  
	If (Jmax.gt.24.) then
		If (float(int(Jmax)).eq.Jmax) then
c			here for integral Jmax
			Jcalc=25.
		elseif (int(Jmax).eq.int(Jmax-.5)) then
c			here for half-integral Jmax
			Jcalc=24.5
		else
			write(6,*) ' stopped in readinfo'
			stop
		endif
	endif
c	write (6,*) ' Jcalc=',Jcalc
c		
c
c	Figure out number of calculations
	calcs=0.
	do 100 Jin=Jmin,Jmax,1.
	do 100 Min=-Jin,Jin,1.
100	calcs=calcs+1.
	calcs=calcs*2.*float(nspect)*float(nangdist)
	calcs=calcs*(Fend-Fstart+1.)*3.*(Ffend-Ffstart+1.)
	calcs=calcs*(2.*Ommaxin+1.)*(2.*Ommaxfin+1.)
c	write to terminal
	write (6,101) calcs
101	format(' ',E9.2,' calculations will be performed')
c	write out step size to terminal
	step=abs(range/float(nspect))
	stepnm=abs((1.e7/startnu)-(1.e7/(startnu+step)))
	if (ifano.eq.0) bpar=dnu
	if (ifano.eq.1) bpar=g
	write (6,697) step,stepnm
	write (6,699) bpar
697	format(' step      = ',f10.3,' cm-1 (',f10.3,' nm)')
699	format(' broadening= ',f10.3,' cm-1')
c
c
c
c	
c
c	
	return
	end
c
c
c		**************************************************
	subroutine writehead
c
	include 'Lookup.f'
	include 'Spectra.f'
	include	'Broad.f'
	include 'ReadCmn.f'
c
	write (7,649) Title1
	write (7,649) Title2
	write (7,750) Sin,Lamin,Pein
	write (7,751) Sfin,Lamfin,Pefin
	write (7,752) Jmin,Jmax
	write (7,753) nangdist,nspect
	write (7,754) ifano
	write (7,755) Bin,Bfin
	write (7,756) Din,Dfin
	write (7,757) SOAin,SOAfin
	write (7,758) Te
	write (7,759) startnu,range
	write (7,760) ifin,iffin
	write (7,761) gamin,gamfin
	write (7,762) llamin,llamfin
	write (7,763) T
	if (ifano.eq.0) then
		write (7,764) dnu
	elseif (ifano.eq.1) then
		write (7,765)
		write (7,766) q, g
	endif
	write (7,770) icntrl, inuc, ihun
	write (7,767)
649	format('/* ',A80)
750	format('/* ','Sin,Lamin,Pein = ',3(F12.8,3x))
751	format('/* ','Sfin,Lamfin,Pefin = ',3(F12.8,3x))
752	format('/* ','Jmin,Jmax = ',2(F12.8,3x))
753	format('/* ','nangdist,nspect = ',2(I5,3x))
754	format('/* ','ifano = ',(I5,3x))
755	format('/* ','Bin,Bfin = ',2(F12.8,3x))
756	format('/* ','Din,Dfin = ',2(F12.8,3x))
757	format('/* ','SOAin,SOAfin = ',2(E14.6,3x))
758	format('/* ','Te = ',(F12.2,3x))
759	format('/* ','startnu,range = ',2(F12.4,3x))
760	format('/* ','ifin,iffin = ',2(I5,3x))
761	format('/* ','gamin,gamfin = ',2(F12.2,3x))
762	format('/* ','llamin,llamfin = ',2(F12.8,3x))
763	format('/* ','Temperature=',F10.2)
764	format('/* ','Lorentzian width dnu  =',(F12.8,3x))
765	format('/* ','Fano Lineshape ')
766	format('/* ','q,  g   = ',2(F12.8,3x))
770	format('/* ','icntrl = ',I3,' inuc = ',I3,' ihun= ',I3)
767	format('/* ',
	&'Lambda(nm)',2x,' Beta   ',2x,'Intensity',2x,'wavenum')
	write(6,*) ' '
c
	return
	end
c
